#include "Define.h"
#include "TileSet.h"
#include "Violations.h"
#include "Adjacent.h"

class HexBoard {
	
	public:

		// The dimensions of the board
		int sizeX, sizeY;
		// The board is a two-dimensional array of integers, 
		// denoting the tile assigned to each location	
		int ** boardTiles;
	   int ** boardOrient;
	
		Violations violations;
		
		int conflict[MAX_TILES];
		int numInConflict;
		
		TileSet * Tiles;
	
		// Constructor: Declares and initialises memory
		HexBoard(int boardSizeX, int boardSizeY, char * tilesToUse);
		
		// Puts a tile on the board at position <x, y> with orientation orient
		boolean PlaceTile(int tile, int orient, int x, int y);
		void DumpBoard();
		// Swaps two tiles
		void SwapTiles(int x1, int y1, int x2, int y2, int no1, int no2);
	
		// Returns the number of violations on the current board
		int CountViolations();

		// Removes all violations for a tile
		void RemoveViolations(int, int);
		
		// Adds violations for a tile
		void UpdateViolations(int, int);
	
		// Creates list of conflicts
		void UpdateConflicts(void);
		
		// Returns boolean true iff two tiles are in violation
		boolean InViolation(int x1, int y1, int x2, int y2);
	
	
		// Returns boolean true iff two locations are adjacent
		boolean IsAdjacent(int x1, int y1, int x2, int y2);
	
		// This function returns tiles adjacent to another tile
		// which is a constant and may be either ALL_ADJACENT to get
		// all tiles or FWD_ADJACENT which is only the first three directions
		Adjacent GetAdjacent(int x, int y, int which);

	private:
		// Retruns true iff a tile <x,y> is part of board
		boolean Valid(int x, int y);
	
};
